{% extends "admin/change_list.html" %}
{% load adminmedia admin_list i18n %}
{% load url from future %}

{% block breadcrumbs %}{% if not is_popup %}
<div class="breadcrumbs">
    {% for breadcrumb in breadcrumbs %}
    {% if breadcrumb.url %}
    <a href="{{breadcrumb.get_absolute_url}}">{{breadcrumb.name}}</a>
    {% else %}
    {{breadcrumb.name}}
    {% endif %}
    {% if not forloop.last %}&rsaquo;{% endif %}
    {% endfor %}
</div>
{% endif %}{% endblock %}

{% block object-tools %}
  {% if has_add_permission %}
    <ul class="object-tools">
      {% block object-tools-items %}
        <li>
          <a href="{{return_link}}">
            {% trans "Return" %}
          </a>
          <a href="{{add_link}}" class="addlink">
            {% blocktrans with cl.opts.verbose_name as name %}Add {{ name }}{% endblocktrans %}
          </a>
        </li>
      {% endblock %}
    </ul>
  {% endif %}
{% endblock %}

{% block result_list %}
  {% result_list cl %}
{% endblock %}
