#from django.views.generic.dates import (ArchiveIndexView, YearArchiveView, MonthArchiveView,
#                                     WeekArchiveView, DayArchiveView, TodayArchiveView,
#                                     DateDetailView)
from detail import DetailView
from edit import DocumentFormMixin, CreateView, UpdateView, DeleteView #FormView
from list import ListView

