{% extends "admin/base_site.html" %}
{% load i18n admin_modify adminmedia %}
{% load url from future %}

{% block extrahead %}{{ block.super }}
{% url 'admin:jsi18n' as jsi18nurl %}
<script type="text/javascript" src="{{ jsi18nurl|default:"../../../jsi18n/" }}"></script>
{{ media }}
{% endblock %}

{% block extrastyle %}{{ block.super }}<link rel="stylesheet" type="text/css" href="{% admin_media_prefix %}css/forms.css" />{% endblock %}

{% block coltype %}{% if ordered_objects %}colMS{% else %}colM{% endif %}{% endblock %}

{% block bodyclass %}{{ opts.app_label }}-{{ opts.object_name.lower }} change-form{% endblock %}

{% block breadcrumbs %}{% if not is_popup %}
<div class="breadcrumbs">
    {% for breadcrumb in breadcrumbs %}
    {% if breadcrumb.url %}
    <a href="{{breadcrumb.get_absolute_url}}">{{breadcrumb.name}}</a>
    {% else %}
    {{breadcrumb.name}}
    {% endif %}
    {% if not forloop.last %}&rsaquo;{% endif %}
    {% endfor %}
</div>
{% endif %}{% endblock %}

{% block content %}<div id="content-main">

<form action="{{ form_url }}" method="get" id="{{ opts.module_name }}_form">{% block form_top %}{% endblock %}
<div>
{% if is_popup %}<input type="hidden" name="_popup" value="1" />{% endif %}
{% if form.errors %}
    <p class="errornote">
    {% blocktrans count errors|length as counter %}Please correct the error below.{% plural %}Please correct the errors below.{% endblocktrans %}
    </p>
    {{ form.non_field_errors }}
{% endif %}
{% for field in adminform.form.hidden_fields %}
{{field}}
{% endfor %}
{% for fieldset in adminform %}
  {% include "admin/includes/fieldset.html" %}
{% endfor %}

{% block submitrow %}
<div class="submit-row" {% if is_popup %}style="overflow: auto;"{% endif %}>
<input type="submit" value="{% trans 'Submit' %}" class="default" {{ onclick_attrib }}/>
</div>
{% endblock %}

</div>
</form></div>
{% endblock %}
